@@ -1,5 +1,5 @@
 package Net::Versa::Director;
-$Net::Versa::Director::VERSION = '0.002000';
+$Net::Versa::Director::VERSION = '0.003000';
 # ABSTRACT: Versa Director REST API client library
 
 use v5.36;
@@ -20,8 +20,13 @@ has 'passwd' => (
     is  => 'rw',
 );
 
-with 'Role::REST::Client';
+has '_refresh_token' => (
+    isa     => Str,
+    is      => 'rw',
+    clearer => 1,
+);
 
+with 'Role::REST::Client';
 
 has '+serializer_class' => (
     default => sub { 'Net::Versa::Director::Serializer' },
@@ -29,6 +34,18 @@ has '+serializer_class' => (
 
 with 'Role::REST::Client::Auth::Basic';
 
+sub _is_oauth ($self) {
+    return $self->server =~ /:9183/;
+}
+
+# has to be after "with 'Role::REST::Client::Auth::Basic';" to be called before
+# its method modifier
+before '_call' => sub ($self, $method, $endpoint, $data, $args) {
+    # disable http basic auth if talking to the OAuth port
+    $args->{authentication} = 'none'
+        if $self->_is_oauth;
+};
+
 has '+persistent_headers' => (
     default => sub {
         return { Accept => 'application/json' };
@@ -53,36 +70,36 @@ sub _error_handler ($self, $res) {
     }
 }
 
-sub _create ($self, $url, $object_data, $query_params = {}, $expected_code = 201) {
+sub _create ($self, $url, $object_data, $query_params = {}, $expected_code = 201, $args = {}) {
     my $params = $self->user_agent->www_form_urlencode( $query_params );
-    my $res = $self->post("$url?$params", $object_data);
+    my $res = $self->post("$url?$params", $object_data, $args);
     $self->_error_handler($res)
         unless $res->code == $expected_code;
 
     return $res->data;
 }
 
-sub _get ($self, $url, $query_params = {}) {
-    my $res = $self->get($url, $query_params);
+sub _get ($self, $url, $query_params = {}, $args = {}) {
+    my $res = $self->get($url, $query_params, $args);
     $self->_error_handler($res)
         unless $res->code == 200;
 
     return $res->data;
 }
 
-sub _update ($self, $url, $object, $object_data, $query_params = {}) {
+sub _update ($self, $url, $object, $object_data, $query_params = {}, $args = {}) {
     my $updated_data = clone($object);
     $updated_data = { %$updated_data, %$object_data };
     my $params = $self->user_agent->www_form_urlencode( $query_params );
-    my $res = $self->put("$url?$params", $updated_data);
+    my $res = $self->put("$url?$params", $updated_data, $args);
     $self->_error_handler($res)
         unless $res->code == 200;
 
     return $res->data;
 }
 
-sub _delete ($self, $url) {
-    my $res = $self->delete($url);
+sub _delete ($self, $url, $args = {}) {
+    my $res = $self->delete($url, undef, $args);
     $self->_error_handler($res)
         unless $res->code == 200;
 
@@ -90,6 +107,31 @@ sub _delete ($self, $url) {
 }
 
 
+sub login ($self, $client_id, $client_secret) {
+    my $login_response = $self->_create('/auth/token', {
+        client_id       => $client_id,
+        client_secret   => $client_secret,
+        username        => $self->user,
+        password        => $self->passwd,
+        grant_type      => "password",
+    }, {}, 200);
+    my $access_token = $login_response->{access_token};
+    $self->set_persistent_header('Authorization', "Bearer $access_token");
+    $self->_refresh_token($login_response->{refresh_token});
+    return $login_response;
+}
+
+
+sub logout ($self) {
+    my $res = $self->_create('/auth/revoke', undef, {}, 200);
+
+    $self->_clear_refresh_token;
+    $self->clear_persistent_headers;
+
+    return $res;
+}
+
+
 sub get_director_info ($self) {
     return $self->_get('/api/operational/system/package-info')
         ->{'package-info'}->[0];
@@ -151,13 +193,15 @@ Net::Versa::Director - Versa Director REST API client library
 
 =head1 VERSION
 
-version 0.002000
+version 0.003000
 
 =head1 SYNOPSIS
 
     use v5.36;
     use Net::Versa::Director;
 
+    # to use the username/password basic authentication
+
     my $director = Net::Versa::Director->new(
         server      => 'https://director.example.com:9182',
         user        => 'username',
@@ -167,6 +211,26 @@ version 0.002000
         },
     );
 
+    # OR to use the OAuth token based authentication
+
+    $director = Net::Versa::Director->new(
+        server      => 'https://director.example.com:9183',
+        user        => 'username',
+        passwd      => '$password',
+        clientattrs => {
+            timeout     => 10,
+        },
+    );
+
+    # this is required to fetch the OAuth access and refresh tokens
+    # using the client id and secret passed to user and passwd.
+    $director->login;
+
+    # at the end of your code, possible in an END block to always execute it
+    # after a successful login to not exceed the maximum number of access
+    # tokens.
+    $director->logout;
+
 =head1 DESCRIPTION
 
 This module is a client library for the Versa Director REST API using the
@@ -179,6 +243,23 @@ L<https://docs.versa-networks.com/Management_and_Orchestration/Versa_Director/Di
 
 =head1 METHODS
 
+=head2 login
+
+Takes a client id and secret.
+
+Logs into the Versa Director when using the OAuth token based port 9183.
+
+Sets the Authorization header to the Bearer access token.
+
+Returns a hashref containing the OAuth access- and refresh-tokens.
+
+=head2 logout
+
+Revokes the access token if OAuth authentication is used so the maximum number
+of access tokens of the client isn't exceeded.
+
+Returns the response.
+
 =head2 get_director_info
 
 Returns the Versa Director information as hashref.
@@ -282,6 +363,18 @@ To run the live API tests the following environment variables need to be set:
 
 =item NET_VERSA_DIRECTOR_PASSWORD
 
+=item NET_VERSA_DIRECTOR_CLIENT_ID
+
+=item NET_VERSA_DIRECTOR_CLIENT_SECRET
+
+=back
+
+If basic authentication tests should be also run set this additional variable to true.
+
+=over
+
+=item NET_VERSA_DIRECTOR_BASIC_AUTH
+
 =back
 
 Only read calls are tested so far.
@@ -292,7 +385,7 @@ Alexander Hartmaier <abraxxa@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2023 by Alexander Hartmaier.
+This software is copyright (c) 2025 by Alexander Hartmaier.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
